// $Id$

/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qfile.h>
#include <qtextstream.h>
#include <qprogressdialog.h>
#include <qclipboard.h>
#include <kinstance.h>
#include <kaction.h>
#include <kstdaction.h>
#include <klistview.h>
#include <klocale.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <ktempfile.h>
#include <kconfigdialog.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kstandarddirs.h>
#include <kiconloader.h>
#include <kapplication.h>
#include <kxmlguifactory.h>
#include <kpopupmenu.h>

#include <bibtexcomment.h>
#include <bibtexpreamble.h>
#include <bibtexstring.h>
#include <bibtexentry.h>
#include <bibtexelement.h>
#include <kbibtexlistviewitem.h>
#include <kbibtexentrywidget.h>
#include <kbibtexcommentwidget.h>
#include <kbibtexstringwidget.h>
#include <kbibtexpreamblewidget.h>
#include <kbibtexfile.h>
#include <kbibtexsettingsdlg.h>
#include "kbibtexlistview.h"

#include <string>

#include <settings.h>
#include "kbibtex_part.h"

Settings kbibtexsettings;

KBibTeXPart::KBibTeXPart( QWidget *parentWidget, const char *widgetName,
                          QObject *parent, const char *name )
        : KParts::ReadWritePart( parent, name ), findPattern( QString::null ), findCaseSensitive( false ), newElementCount( 0 ), findDlg( widget(), "find_dlg" ), m_bibtexfile( new KBibTeXFile() ), settingsDlg( NULL )
{
    // we need an instance
    setInstance( KBibTeXPartFactory::instance() );

    // this should be your custom internal widget
    m_bibtexlist = new KBibTeXListView( this, parentWidget, widgetName );

    // notify the part that this is our internal widget
    setWidget( m_bibtexlist );

    // set our XML-UI resource file
    setXMLFile( "kbibtex_part.rc" );

    // setup GUI elements
    setupGUI();

    // create our actions
    setupActions();

    // we are read-write by default
    setReadWrite( true );

    // we are not modified since we haven't done anything yet
    setModified( false );

    // read configuration
    readSettings();

    updateMenu();
}

KBibTeXPart::~KBibTeXPart()
{
    // save configuration
    writeSettings();

    if ( !settingsDlg )
        delete settingsDlg;
}

void KBibTeXPart::setReadWrite( bool rw )
{
    // notify your internal widget of the read-write state
    /*
       m_bibtexlist->setReadOnly(!rw);
       if (rw)
           connect(m_bibtexlist, SIGNAL(textChanged()),
                   this,     SLOT(setModified()));
       else
       {
           disconnect(m_bibtexlist, SIGNAL(textChanged()),
                      this,     SLOT(setModified()));
       }
    */

    ReadWritePart::setReadWrite( rw );
}

void KBibTeXPart::setModified( bool modified )
{
    // get a handle on our Save action and make sure it is valid
    KAction * save = actionCollection() ->action( KStdAction::stdName( KStdAction::Save ) );
    if ( !save )
        return ;

    // if so, we either enable or disable it based on the current
    // state
    if ( modified )
        save->setEnabled( true );
    else
        save->setEnabled( false );

    // in any event, we want our parent to do it's thing
    ReadWritePart::setModified( modified );
}

bool KBibTeXPart::openFile()
{
    bool result = false;
    delete m_bibtexfile;

    m_bibtexfile = new KBibTeXFile();

    KTempFile tempFile;
#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
    if ( KIO::NetAccess::file_copy( m_url, KURL( tempFile.name() ), -1, true, false, widget() ) )
#else
    if ( KIO::NetAccess::download( m_url, tempFile.name() ) )
#endif
    {
        progressDialog = new QProgressDialog( widget() );
        progressDialog->setLabelText( QString( i18n( "<qt>Parsing file <b>%1</b></qt>" ) ).arg( m_url.prettyURL() ) );
        connect( m_bibtexfile, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
        connect( this, SIGNAL( progressCancel() ), m_bibtexfile, SLOT( cancelOperation( ) ) );
        QApplication::setOverrideCursor( Qt::waitCursor );
        result = m_bibtexfile->load( tempFile.name() );

        disconnect( m_bibtexfile, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
        disconnect( this, SIGNAL( progressCancel() ), m_bibtexfile, SLOT( cancelOperation( ) ) );
        delete progressDialog;
        progressDialog = NULL;

        if ( result )
        {
            m_bibtexlist->clear();
            for ( unsigned int i = 0; i < m_bibtexfile->count(); i++ )
                new KBibTeXListViewItem( m_bibtexfile->at( i ), m_bibtexlist );

            recent->addURL( m_url );

            // just for fun, set the status bar
            emit setStatusBarText( m_url.prettyURL() );

            updateMenu();

            int width = m_bibtexlist->width();
            m_bibtexlist->setColumnWidth( 0, width / 15 );
            m_bibtexlist->setColumnWidth( 1, width * 2 / 15 );
            m_bibtexlist->setColumnWidth( 2, width * 6 / 15 );
            m_bibtexlist->setColumnWidth( 3, width * 4 / 15 );
            m_bibtexlist->setColumnWidth( 4, width / 15 );
        }
        QApplication::restoreOverrideCursor();
    }

    if ( !result )
        KMessageBox::error( NULL, i18n( "Cannot access file\n%1" ).arg( m_url.prettyURL() ), i18n( "File Open Error" ) );

    return result;
}

bool KBibTeXPart::saveFile()
{
    bool result = false;

    if ( m_url.isValid() )
    {
        if ( isReadWrite() )
        {
            BibTeXFile::Encoding encoding;
            switch ( kbibtexsettings.encoding )
            {
            case Settings::encUTF8: encoding = BibTeXFile::encUTF8; break;
            default:
                encoding = BibTeXFile::encLaTeX;
            }

            m_bibtexfile->reorderElements( m_bibtexlist );
            KTempFile tempFile;
            m_bibtexfile->save( tempFile.name(), encoding );
#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
            tempFile.sync();
#endif

#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
            result = KIO::NetAccess::file_copy( KURL( tempFile.name() ), m_url, -1, true, false, widget() );
#else
            result = KIO::NetAccess::upload( tempFile.name(), m_url );
#endif
            if ( result )
            {
                setModified( false );
                recent->addURL( m_url );
                emit setStatusBarText( m_url.prettyURL() );
            }

            tempFile.unlink();
        }

        if ( ! result )
            KMessageBox::error( NULL, i18n( "Cannot save file\n%1" ).arg( m_url.prettyURL() ), i18n( "File Save Error" ) );
    }
    else
        fileSaveAs();

    return result;
}

void KBibTeXPart::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
    KURL url = KFileDialog::getOpenURL( QString::null, "*.bib|" + i18n( "BibTeX Files (*.bib)" ) );

    if ( url.isEmpty() == false )
        openURL( url );
}

void KBibTeXPart::fileSaveAs()
{
    // this slot is called whenever the File->Save As menu is selected,
    KURL url = KFileDialog::getSaveURL( QString::null, "*.bib|" + i18n( "BibTeX Files (*.bib)" ) );
    if ( ! url.isEmpty() )
    {
        saveAs( url );
        recent->addURL( url );
    }
}


// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance* KBibTeXPartFactory::s_instance = 0L;
KAboutData* KBibTeXPartFactory::s_about = 0L;

KBibTeXPartFactory::KBibTeXPartFactory()
        : KParts::Factory()
{}

KBibTeXPartFactory::~KBibTeXPartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KBibTeXPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
        QObject *parent, const char *name,
        const char *classname, const QStringList &args )
{
    // Create an instance of our Part
    KBibTeXPart * obj = new KBibTeXPart( parentWidget, widgetName, parent, name );

    // See if we are to be read-write or not
    if ( QCString( classname ) == "KParts::ReadOnlyPart" )
        obj->setReadWrite( false );

    return obj;
}

KInstance* KBibTeXPartFactory::instance()
{
    if ( !s_instance )
    {
        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "20050521" );
        s_about->addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
        s_instance = new KInstance( s_about );
    }
    return s_instance;
}

extern "C"
{
    void* init_libkbibtexpart()
    {
        return new KBibTeXPartFactory;
    }
};

void KBibTeXPart::setupGUI()
{
    //TODO
}

void KBibTeXPart::setupActions()
{
    KStdAction::open( this, SLOT( fileOpen() ), actionCollection() );
    KStdAction::saveAs( this, SLOT( fileSaveAs() ), actionCollection() );
    KStdAction::save( this, SLOT( save() ), actionCollection() );
    actionEditCut = KStdAction::cut( this, SLOT( cutElement() ), actionCollection() );
    actionEditCopy = KStdAction::copy( this, SLOT( copyElement() ), actionCollection() );
    actionEditPaste = KStdAction::paste( this, SLOT( pasteElement() ), actionCollection() );

    actionEditFind = KStdAction::find( this, SLOT( slotFind() ), actionCollection() );
    actionEditFindNext = KStdAction::findNext( this, SLOT( slotFindNext() ), actionCollection() );
    actionEditFindPrev = KStdAction::findPrev( this, SLOT( slotFindPrev() ), actionCollection() );
    actionPreferences = KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );

    actionEditSearchInternetGoogle = new KAction( i18n( "Search &Google" ), 0, this, SLOT( slotSearchGoogle() ), actionCollection(), "edit_searchinternet_google" );
    actionEditSearchInternetGoogleScholar = new KAction( i18n( "Search Google &Scholar" ), 0, this, SLOT( slotSearchGoogleScholar() ), actionCollection(), "edit_searchinternet_googlescholar" );
    actionEditSearchInternetCiteseer = new KAction( i18n( "Search &CiteSeer" ), 0, this, SLOT( slotSearchCiteseer() ), actionCollection(), "edit_searchinternet_citeseer" );
    actionFileExportPDF = new KAction( i18n( "Export as &PDF..." ), 0, this, SLOT( slotExportPDF () ), actionCollection(), "file_export_pdf" );
    actionFileExportPS = new KAction( i18n( "Export as &PS..." ), 0, this, SLOT( slotExportPS() ), actionCollection(), "file_export_ps" );
    actionEditSearchInternetCiteseer = new KAction( i18n( "Search &CiteSeer" ), 0, this, SLOT( slotSearchCiteseer() ), actionCollection(), "edit_searchinternet_citeseer" );
    //     actionPreferences = new KAction( i18n( "Preferences" ), 0, this, SLOT( slotPreferences() ), actionCollection(), "edit_preferences" );

    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etArticle, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_article" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etBook, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_book" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etBooklet, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_booklet" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etConference, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_conference" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etInBook, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_inbook" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etInCollection, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_incollection" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etInProceedings, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_inproceedings" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etManual, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_manual" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etMastersThesis, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_mastersthesis" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etMisc, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_misc" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etPhDThesis, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_phdthesis" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etProceedings, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_proceedings" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etTechReport, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_techreport" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etUnpublished, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_entry_unpublished" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etComment, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_comment" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etPreamble, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_preamble" );
    ( void ) new KAction( BibTeXElement::elementTypeDescriptionI18N( BibTeXElement::etString, true ), 0, this, SLOT( newElement() ), actionCollection(), "element_new_string" );
    actionEditElement = new KAction( i18n( "&Edit" ), 0, this, SLOT( editElement() ), actionCollection(), "element_edit" );
    actionDeleteElement = new KAction( i18n( "&Delete" ), 0, this, SLOT( deleteElement() ), actionCollection(), "element_delete" );

    recent = new KRecentFilesAction( i18n( "Open &Recent Files" ), 0,
                                     this, SLOT( openURL( const KURL & ) ),
                                     actionCollection(), "file_open_recent" );

    connect( m_bibtexlist, SIGNAL( executed( QListViewItem* ) ), this, SLOT( editElement( QListViewItem* ) ) );
    connect( m_bibtexlist, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged( ) ) );
    connect( m_bibtexlist, SIGNAL( contextMenu( KListView *, QListViewItem *, const QPoint & ) ), this, SLOT( showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint & ) ) );
}

void KBibTeXPart::newElement()
{
    BibTeXElement * newElement = NULL;

    if ( sender() )
    {
        QString newId = QString( i18n( "NewElement_%1" ) ).arg( newElementCount++ );

        if ( strncmp( sender() ->name(), "element_new_entry_", 18 ) == 0 )
        {
            BibTeXElement::ElementType elementType = BibTeXElement::elementTypeFromString( sender() ->name() + 18 );

            if ( elementType != BibTeXElement::etUnknown )
                newElement = new BibTeXEntry( elementType, newId );
        }
        else if ( strcmp( sender() ->name(), "element_new_comment" ) == 0 )
        {
            newElement = new BibTeXComment( i18n( "Empty comment" ) );
        }
        else if ( strcmp( sender() ->name(), "element_new_preamble" ) == 0 )
        {
            newElement = new BibTeXPreamble( i18n( "Empty preamble" ) );
        }
        else if ( strcmp( sender() ->name(), "element_new_string" ) == 0 )
        {
            newElement = new BibTeXString( newId, i18n( "Empty string" ) );
        }

        if ( newElement != NULL )
        {
            KBibTeXListViewItem * lvi = new KBibTeXListViewItem( newElement, m_bibtexlist );
            if ( editElement( lvi ) )
                m_bibtexfile->appendElement( newElement );
            else
            {
                delete lvi;
                delete newElement;
            }
        }
    }
}

bool KBibTeXPart::editElement( QListViewItem * item )
{
    QDialog::DialogCode dialogResult = QDialog::Rejected;

    if ( item == NULL )
        item = m_bibtexlist->selectedItem();

    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( item );
    if ( kblvi )
    {
        BibTeXEntry * entry = dynamic_cast<BibTeXEntry*>( kblvi->element() );
        if ( entry )
            dialogResult = KBibTeXEntryWidget::execute( entry, m_bibtexfile, i18n( "Edit BibTeX Entry" ) );
        else
        {
            BibTeXComment* comment = dynamic_cast<BibTeXComment*>( kblvi->element() );
            if ( comment )
                dialogResult = KBibTeXCommentWidget::execute( comment, i18n( "Edit BibTeX Comment" ) );
            else
            {
                BibTeXString* string = dynamic_cast<BibTeXString*>( kblvi->element() );
                if ( string )
                    dialogResult = KBibTeXStringWidget::execute( string, i18n( "Edit BibTeX String" ) );
                else
                {
                    BibTeXPreamble* preamble = dynamic_cast<BibTeXPreamble*>( kblvi->element() );
                    if ( preamble )
                        dialogResult = KBibTeXPreambleWidget::execute( preamble, i18n( "Edit BibTeX Preamble" ) );
                }
            }
        }

        if ( dialogResult == QDialog::Accepted )
        {
            kblvi->updateItem();
            setModified( true );
        }
    }

    return dialogResult == QDialog::Accepted;
}

void KBibTeXPart::deleteElement()
{
    if ( m_bibtexlist->selectedItem() != NULL )
    {
        BibTeXElement * element = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->selectedItem() ) ->element();
        if ( element )
            m_bibtexfile->deleteElement( element );
        delete m_bibtexlist->selectedItem();
        setModified( true );
    }
}

void KBibTeXPart::showProgress( int current, int total )
{
    if ( progressDialog != NULL )
    {
        progressDialog->setTotalSteps( total );
        progressDialog->setProgress( current );
#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
        if ( progressDialog->wasCanceled() )
            emit progressCancel();
#endif

    }

    kapp->processEvents();
}

/*!
    \fn KBibTeXPart::readSettings()
 */
void KBibTeXPart::readSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    recent->loadEntries( config );

    kbibtexsettings.load( config );
}

/*!
    \fn KBibTeXPart::writeSettings()
 */
void KBibTeXPart::writeSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    recent->saveEntries( config );

    kbibtexsettings.save( config );
}

/*!
    \fn KBibTeXPart::cutElement()
 */
void KBibTeXPart::cutElement()
{
#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
    QString buffer;
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );

    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );

        buffer.append( kblvi->element() ->toBibTeXString() );

        delete it.current();
        it++;
    }
#endif

}

/*!
    \fn KBibTeXPart::copyElement()
 */
void KBibTeXPart::copyElement()
{
#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
    QString buffer;
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );

    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );

        buffer.append( kblvi->element() ->toBibTeXString() );

        it++;
    }

    kapp->clipboard() ->setText( buffer );
#endif

}

/*!
    \fn KBibTeXPart::pasteElement()
 */
void KBibTeXPart::pasteElement()
{
    QString clipText( kapp->clipboard() ->text() );
    insertText( clipText );
}

void KBibTeXPart::showBibtexListContextMenu( KListView *l, QListViewItem *i, const QPoint &p )
{
    if ( factory() )
    {
        updateMenu();
        static_cast<KPopupMenu*>( factory() ->container( "popup_bibtexlist", this ) ) ->popup( p );
    }
}

void KBibTeXPart::updateMenu()
{
    QListViewItem * lvi = m_bibtexlist->selectedItem();

    actionEditElement->setEnabled( lvi != NULL );
    actionDeleteElement->setEnabled( lvi != NULL );
    actionEditCut->setEnabled( lvi != NULL );
    actionEditCopy->setEnabled( lvi != NULL );
    actionEditSearchInternetGoogle->setEnabled( lvi != NULL );
    actionEditSearchInternetGoogleScholar->setEnabled( lvi != NULL );
    actionEditSearchInternetCiteseer->setEnabled( lvi != NULL );
}

void KBibTeXPart::slotSelectionChanged()
{
    updateMenu();
}

void KBibTeXPart::insertText( QString &text )
{
    BibTeXFile * bibtexfile = new BibTeXFile();
    QTextStream textStream( text, IO_ReadOnly );
    bibtexfile->load( textStream );

    for ( unsigned int i = 0; i < bibtexfile->count(); i++ )
    {
        new KBibTeXListViewItem( bibtexfile->at( i ), m_bibtexlist );
        m_bibtexfile->appendElement( bibtexfile->at( i ) );
    }

    delete bibtexfile;

    setModified( true );
}

void KBibTeXPart::slotFind()
{
    findDlg.setOptions( KFindDialog::FromCursor );
    findDlg.setHasSelection( false );
    findDlg.setHasCursor( m_bibtexlist->currentItem( ) != NULL );
#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,92)
    findDlg.setSupportsBackwardsFind( true );
    findDlg.setSupportsWholeWordsFind( false );
    findDlg.setSupportsRegularExpressionFind( false );
#endif
    if ( findDlg.exec() == QDialog::Accepted )
    {
        findPattern = findDlg.pattern();
        findCaseSensitive = ( findDlg.options() && KFindDialog::CaseSensitive ) != 0;
        SearchDirection direction = ( findDlg.options() && KFindDialog::FindBackwards ) == 0 ? sdForward : sdBackward;
        QListViewItem* startAt = ( findDlg.options() && KFindDialog::FromCursor ) == 0 ? m_bibtexlist->firstChild() : ( m_bibtexlist->selectedItem( ) == NULL ? m_bibtexlist->currentItem( ) : m_bibtexlist->selectedItem( ) );
        find( startAt, direction );
    }
}

void KBibTeXPart::slotFindPrev()
{
    if ( findPattern == QString::null )
        slotFind();
    else
        find( m_bibtexlist->currentItem(), sdBackward );
}

void KBibTeXPart::slotFindNext()
{
    if ( findPattern == QString::null )
        slotFind();
    else
        find( m_bibtexlist->currentItem(), sdForward );
}

void KBibTeXPart::find( QListViewItem *startAt, SearchDirection direction )
{
    if ( startAt != NULL )
    {
        KBibTeXListViewItem * start = dynamic_cast<KBibTeXListViewItem*>( startAt );
        KBibTeXListViewItem * cur = start;
        bool found = false;
        do
        {
            if ( direction == sdForward )
            {
                cur = dynamic_cast<KBibTeXListViewItem*>( cur->itemBelow() );
                if ( cur == NULL )
                    // wrap around
                    cur = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->firstChild() );
            }
            else
            {
                cur = dynamic_cast<KBibTeXListViewItem*>( cur->itemAbove() );
                if ( cur == NULL )
                    // wrap around
                    cur = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->lastItem() );
            }

            if ( cur )
                found = cur->element() ->containsPattern( findPattern, findCaseSensitive );
        }
        while ( !found && ( cur != NULL ) && ( cur != start ) );

        if ( found )
        {
            if ( cur == start )
                KMessageBox::information( widget(), i18n( "No more occurences found." ), i18n( "Not found" ) );
            else
            {
                m_bibtexlist->setCurrentItem( cur );
                m_bibtexlist->setSelected( cur, true );
                m_bibtexlist->ensureItemVisible( cur );
            }
        }
        else
            KMessageBox::information( widget(), i18n( "The search string was not found." ), i18n( "Not found" ) );
    }
}

void KBibTeXPart::slotSearchGoogle()
{
    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->selectedItem() );
    if ( kblvi != NULL )
        searchInternet( kblvi->element(), iseGoogle );
}

void KBibTeXPart::slotSearchGoogleScholar()
{
    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->selectedItem() );
    if ( kblvi != NULL )
        searchInternet( kblvi->element(), iseGoogleScholar );
}

void KBibTeXPart::slotSearchCiteseer()
{
    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( m_bibtexlist->selectedItem() );
    if ( kblvi != NULL )
        searchInternet( kblvi->element(), iseCiteSeer );
}

void KBibTeXPart::searchInternet( BibTeXElement *element, InternetSearchEngine engine )
{
    QString queryString = QString::null;

    BibTeXEntry* entry = dynamic_cast<BibTeXEntry*>( element );
    if ( entry != NULL )
        queryString = *( entry->field( BibTeXEntry::ftTitle ) ->value );
    else
    {
        BibTeXComment* comment = dynamic_cast<BibTeXComment*>( element );
        if ( comment != NULL )
            queryString = comment->comment();
        else
        {
            BibTeXString* string = dynamic_cast<BibTeXString*>( element );
            if ( string != NULL )
                queryString = string->value();
            else
                qDebug( "Not yet supported" );
        }
    }

    if ( queryString != QString::null )
        switch ( engine )
        {
        case iseGoogle: kapp->invokeBrowser( QString( "http://www.google.com/search?q=%1" ).arg( queryString ) );
            break;
        case iseGoogleScholar: kapp->invokeBrowser( QString( "http://scholar.google.com/scholar?q=%1" ).arg( queryString ) );
            break;
        case iseCiteSeer: kapp->invokeBrowser( QString( "http://citeseer.ist.psu.edu/cs?q=%1&submit=Search+Documents&cs=1" ).arg( queryString ) );
            break;
        }
}

void KBibTeXPart::slotExportPDF()
{
    if ( KMessageBox::questionYesNo( widget(), i18n( "This process will most likely fail if the BibTeX file is not well-formed or relies on additional packages.\n\nContinue anyway?" ), i18n( "Export" ) ) == KMessageBox::Yes )
    {
        KURL url = KFileDialog::getSaveURL( QString::null, i18n( "*.pdf|Portable Document Format document" ), widget(), i18n( "Export as PDF" ) );
        if ( url.isValid() && !m_bibtexfile->exportAs( KBibTeXFile::ftPDF, url, widget() ) )
            KMessageBox::error( widget(), i18n( "Exporting to PDF failed" ), i18n( "Export Error" ) );
    }
}

void KBibTeXPart::slotExportPS()
{
    if ( KMessageBox::questionYesNo( widget(), i18n( "This process will most likely fail if the BibTeX file is not well-formed or relies on additional packages.\n\nContinue anyway?" ), i18n( "Export" ) ) == KMessageBox::Yes )
    {
        KURL url = KFileDialog::getSaveURL( QString::null, i18n( "*.ps|PostScript document" ), widget(), i18n( "Export as PS" ) );
        if ( url.isValid() && !m_bibtexfile->exportAs( KBibTeXFile::ftPS, url, widget() ) )
            KMessageBox::error( widget(), i18n( "Exporting to PostScript failed" ), i18n( "Export Error" ) );
    }
}

void KBibTeXPart::slotPreferences()
{
    if ( !settingsDlg )
        settingsDlg = new KBibTeXSettingsDlg( widget() );
    settingsDlg->show();
}

#include "kbibtex_part.moc"

