/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qapplication.h>
#include <qprogressdialog.h>
#include <qprocess.h>
#include <qfile.h>
#include <qwaitcondition.h>
#include <kurl.h>
#include <ktempdir.h>
#include <kio/netaccess.h>
#include <klocale.h>
#include <kbibtexlistview.h>
#include <kbibtexlistviewitem.h>

#include "kbibtexfile.h"

KBibTeXFile::KBibTeXFile( QObject *parent, const char *name )
        : BibTeXFile( parent, name )
{
    wc = new QWaitCondition();
}


KBibTeXFile::~KBibTeXFile()
{
    delete wc;
}

void KBibTeXFile::reorderElements( KBibTeXListView *lv )
{
    KBibTeXListViewItem * lvCur = dynamic_cast<KBibTeXListViewItem*>( lv->firstChild() );
    KBibTeXListViewItem * nextLvCur = NULL;

    elements.clear();
    while ( lvCur )
    {
        elements.append( lvCur->element() );
        nextLvCur = dynamic_cast<KBibTeXListViewItem*>( lvCur->firstChild() );
        if ( nextLvCur == NULL )
        {
            nextLvCur = dynamic_cast<KBibTeXListViewItem*>( lvCur->nextSibling() );
            if ( nextLvCur == NULL )
                nextLvCur = lvCur->parent() == NULL ? NULL : dynamic_cast<KBibTeXListViewItem*>( lvCur->parent() ->nextSibling() );
        }

        lvCur = nextLvCur;
    }
}

bool KBibTeXFile::exportAs( KBibTeXFile::FileType fileType, KURL& filename, QWidget*window )
{
    switch ( fileType )
    {
    case ftPDF:
        return exportAsPDF( filename, window );
        break;
    case ftPS:
        return exportAsPS( filename, window );
        break;
    default:
        return false;
    }
}

bool KBibTeXFile::exportAsPS( KURL& filename, QWidget*window )
{
    KTempDir tempDir = KTempDir();
    tempDir.setAutoDelete( true );
    QString LaTeXFilename = QString( tempDir.name() ).append( "kbibtex.tex" );
    QString BibTeXFilename = QString( tempDir.name() ).append( "kbibtex.bib" );
    QString PSFilename = QString( tempDir.name() ).append( "kbibtex.ps" );

    QStringList cmdLines = QStringList::split( '|', "latex kbibtex.tex|bibtex kbibtex|latex kbibtex.tex|latex kbibtex.tex|dvips -o kbibtex.ps kbibtex.dvi" );
    return save( BibTeXFilename ) && writeLatexFile( LaTeXFilename ) && runProcesses( *tempDir.qDir(), cmdLines, window ) && KIO::NetAccess::upload( PSFilename, filename, window );
}

bool KBibTeXFile::exportAsPDF( KURL& filename, QWidget*window )
{
    KTempDir tempDir = KTempDir();
    tempDir.setAutoDelete( true );
    QString LaTeXFilename = QString( tempDir.name() ).append( "kbibtex.tex" );
    QString BibTeXFilename = QString( tempDir.name() ).append( "kbibtex.bib" );
    QString PDFFilename = QString( tempDir.name() ).append( "kbibtex.pdf" );

    QStringList cmdLines = QStringList::split( '|', "pdflatex kbibtex.tex|bibtex kbibtex|pdflatex kbibtex.tex|pdflatex kbibtex.tex" );
    return save( BibTeXFilename ) && writeLatexFile( LaTeXFilename ) && runProcesses( *tempDir.qDir(), cmdLines , window ) && KIO::NetAccess::upload( PDFFilename, filename, window );
}

bool KBibTeXFile::writeLatexFile( QString &filename )
{
    QFile latexFile( filename );
    if ( latexFile.open( IO_WriteOnly ) )
    {
        QTextStream ts( &latexFile );
        ts << "\\documentclass{article}\n";
        ts << "\\usepackage[" << kbibtexsettings.exportlanguage << "]{babel}\n";
        ts << "\\usepackage{url}\n";
        ts << "\\bibliographystyle{" << kbibtexsettings.exportbibliographystyle << "}\n";
        ts << "\\begin{document}\n";
        ts << "\\nocite{*}\n";
        ts << "\\bibliography{kbibtex}\n";
        ts << "\\end{document}\n";
        latexFile.close();
        return true;
    }
    else return false;

}

void KBibTeXFile::slotProcessExited()
{
    wc->wakeAll();
}

void KBibTeXFile::slotCancelProcess()
{
    process->tryTerminate ();
    process->kill ();
}

bool KBibTeXFile::runProcess( const QDir& tempDir, const QStringList &args )
{
    bool result = true;

    if ( process == NULL )
    {
        process = new QProcess( );
        connect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        process->setWorkingDirectory( tempDir );
        process->setArguments( args );

        if ( ( result = process->start() ) )
        {
            qApp->processEvents();
            while ( process->isRunning() )
            {
                wc->wait( 250 );
                qApp->processEvents();
            }

            result = process->normalExit();
        }

        delete( process );
    }

    process = NULL;
    return result;
}

bool KBibTeXFile::runProcesses( const QDir& tempDir, const QStringList &commandLines, QWidget*window )
{
    bool result = true;
    QApplication::setOverrideCursor( Qt::waitCursor );
    QProgressDialog *progressDialog = new QProgressDialog( window );
    progressDialog->setLabelText( i18n( "Exporting..." ) );
    connect( progressDialog, SIGNAL( canceled () ), this, SLOT( slotCancelProcess() ) );

    int i = 0;
    progressDialog->setProgress( ++i, commandLines.size() );
    for ( QStringList::ConstIterator it = commandLines.begin(); result && !progressDialog->wasCanceled() && it != commandLines.end(); it++ )
    {
        QStringList args = QStringList::split( ' ', *it );
        result &= runProcess( tempDir, args );
        progressDialog->setProgress( ++i, commandLines.size() );
        qApp->processEvents();
    }

    delete progressDialog;
    QApplication::restoreOverrideCursor();
    return result;
}

#include "kbibtexfile.moc"
